require File.expand_path("../cre_property_matcher/version.rb", __FILE__)
require File.expand_path("../cre_property_matcher/property_data_loader.rb", __FILE__)
require File.expand_path("../cre_property_matcher/property_data_calculator.rb", __FILE__)

module CrePropertyMatcher
	class PropertyMatcher
		include PropertyDataCalculator

		attr_accessor :reference_property, :comparison_properties

		def initialize(reference_property)
			@reference_property = Property.new(reference_property)
			@comparison_properties = PropertyDataLoader.run_and_return_properties
			@features = [:@appraised_value, :@noi, :@ncf, :@uw_revenue, :@uw_expenses]
			@scales = scale_features
		end

		def reference
			@reference_property
		end

		def comps
			@comparison_properties
		end

		def get_suitable_comps
			suitable = []
			
			comps.each do |property|
				all_features_not_nil = true
				features.each do |feature|
					unless property.instance_variable_get(feature)
						all_features_not_nil = false
					end
				end
				if all_features_not_nil
					suitable << property
				end
			end

			suitable
		end

		def features
			@features
		end

		def scales
			@scales
		end
		
		def head_comparison_properties(n = 6)
			result = []
			1.upto(n) do |index|
				result << comps[index]
			end
			result
		end

		def property_type_count
			count_hash = Hash.new(0)
			@comparison_properties.each do |property|
				count_hash[property.general_property_type] += 1
			end
			count_hash
		end

		def filter_by_feature(feature)
			result = []
			comps.each do |property|
				result << property.instance_variable_get(feature)
			end
			result
		end

		def scale_features
			scales = {}
			features.each do |feature|
				array = filter_by_feature(feature).compact
		
				# p array

				scales[feature] = {min: array.min, max: array.max, range: array.max - array.min}
			end
			scales
		end

		def calculate_distances
			props = get_suitable_comps
			reference.calculate_distance_from_neighbors(props, features, scales)
		end
		
		def sort_comps_by_distance
			comps.sort { |a, b| a.distance <=> b.distance }
		end

		def get_k_nearest(k = 3)
			sort_comps_by_distance.take(k)
		end
	end
end
